#include "stdafx.h"
#include "JTAPIWrapper.h"

/* Defines */
#define JTAPI_FUNC_CLOSEPLUGIN		_T("JTAPI_ClosePlugin")
#define JTAPI_FUNC_INIREADSECTION	_T("JTAPI_IniReadSection")
#define	JTAPI_FUNC_INIREADINT		_T("JTAPI_IniReadInt")
#define JTAPI_FUNC_INIREADSTRING	_T("JTAPI_IniReadString")
#define JTAPI_FUNC_INIREADBOOL		_T("JTAPI_IniReadBool")
#define JTAPI_FUNC_INIWRITESECTION	_T("JTAPI_IniWriteSection")
#define JTAPI_FUNC_INIWRITEINT		_T("JTAPI_IniWriteInt")
#define JTAPI_FUNC_INIWRITESTRING	_T("JTAPI_IniWriteString")
#define JTAPI_FUNC_INIWRITEBOOL		_T("JTAPI_IniWriteBool")

/* Typedefs */
typedef void (WINAPI *PFNJTAPI_ClosePlugin)(HMODULE hModule);
typedef DWORD (WINAPI *PFNJTAPI_IniReadSection)(HMODULE hModule, LPWSTR lpReturnedString, DWORD nSize);
typedef int (WINAPI *PFNJTAPI_IniReadInt)(HMODULE hModule, LPCWSTR lpKeyName, int nDefault);
typedef DWORD (WINAPI *PFNJTAPI_IniReadString)(HMODULE hModule, LPCWSTR lpKeyName, LPCWSTR lpDefault, LPWSTR lpReturnedString, DWORD nSize);
typedef BOOL (WINAPI *PFNJTAPI_IniReadBool)(HMODULE hModule, LPCWSTR lpKeyName, BOOL fDefault);
typedef BOOL (WINAPI *PFNJTAPI_IniWriteSection)(HMODULE hModule, LPCWSTR lpString);
typedef BOOL (WINAPI *PFNJTAPI_IniWriteInt)(HMODULE hModule, LPCWSTR lpKeyName, int nValue);
typedef BOOL (WINAPI *PFNJTAPI_IniWriteString)(HMODULE hModule, LPCWSTR lpKeyName, LPCWSTR lpString);
typedef BOOL (WINAPI *PFNJTAPI_IniWriteBool)(HMODULE hModule, LPCWSTR lpKeyName, BOOL fValue);

typedef struct _JTAPIENTRY {
	PFNJTAPI_ClosePlugin		pfnClosePlugin;
	PFNJTAPI_IniReadSection		pfnIniReadSection;
	PFNJTAPI_IniReadInt			pfnIniReadInt;
	PFNJTAPI_IniReadString		pfnIniReadString;
	PFNJTAPI_IniReadBool		pfnIniReadBool;
	PFNJTAPI_IniWriteSection	pfnIniWriteSection;
	PFNJTAPI_IniWriteInt		pfnIniWriteInt;
	PFNJTAPI_IniWriteString		pfnIniWriteString;
	PFNJTAPI_IniWriteBool		pfnIniWriteBool;
} JTAPIENTRY;

/* Global Variables */
static	HMODULE		gs_hSelf		= NULL;
static	BOOL		gs_bInitialized	= FALSE;
static	JTAPIENTRY	gs_Entry;



BOOL
JTAPIWrap_Initialize(
	HMODULE	hJoeTune,
	HMODULE	hSelf
)
{
	BOOL		bRet;
	JTAPIENTRY	apiEntry;

	bRet = FALSE;

	if (hJoeTune == NULL || hSelf == NULL) {
		return FALSE;
	}

	if (gs_bInitialized) {
		return FALSE;
	}

	memset(&apiEntry, 0, sizeof(apiEntry));

	apiEntry.pfnClosePlugin		= (PFNJTAPI_ClosePlugin)	GetProcAddress(hJoeTune, JTAPI_FUNC_CLOSEPLUGIN);
	apiEntry.pfnIniReadSection	= (PFNJTAPI_IniReadSection)	GetProcAddress(hJoeTune, JTAPI_FUNC_INIREADSECTION);
	apiEntry.pfnIniReadInt		= (PFNJTAPI_IniReadInt)		GetProcAddress(hJoeTune, JTAPI_FUNC_INIREADINT);
	apiEntry.pfnIniReadString	= (PFNJTAPI_IniReadString)	GetProcAddress(hJoeTune, JTAPI_FUNC_INIREADSTRING);
	apiEntry.pfnIniReadBool		= (PFNJTAPI_IniReadBool)	GetProcAddress(hJoeTune, JTAPI_FUNC_INIREADBOOL);
	apiEntry.pfnIniWriteSection	= (PFNJTAPI_IniWriteSection)GetProcAddress(hJoeTune, JTAPI_FUNC_INIWRITESECTION);
	apiEntry.pfnIniWriteInt		= (PFNJTAPI_IniWriteInt)	GetProcAddress(hJoeTune, JTAPI_FUNC_INIWRITEINT);
	apiEntry.pfnIniWriteString	= (PFNJTAPI_IniWriteString)	GetProcAddress(hJoeTune, JTAPI_FUNC_INIWRITESTRING);
	apiEntry.pfnIniWriteBool	= (PFNJTAPI_IniWriteBool)	GetProcAddress(hJoeTune, JTAPI_FUNC_INIWRITEBOOL);

	if (apiEntry.pfnClosePlugin		!= NULL &&
		apiEntry.pfnIniReadSection	!= NULL &&
		apiEntry.pfnIniReadInt		!= NULL &&
		apiEntry.pfnIniReadString	!= NULL &&
		apiEntry.pfnIniReadBool		!= NULL &&
		apiEntry.pfnIniWriteSection	!= NULL &&
		apiEntry.pfnIniWriteInt		!= NULL &&
		apiEntry.pfnIniWriteString	!= NULL &&
		apiEntry.pfnIniWriteBool	!= NULL	) {

		memcpy(&gs_Entry, &apiEntry, sizeof(JTAPIENTRY));
		gs_hSelf	= hSelf;

		gs_bInitialized = TRUE;

		bRet = TRUE;
	}

	return bRet;
}


BOOL
JTAPIWrap_Terminate()
{
	BOOL	bRet;

	bRet = FALSE;

	if (gs_bInitialized) {

		gs_hSelf	= NULL;
		memset(&gs_Entry, 0, sizeof(gs_Entry));

		gs_bInitialized = FALSE;

		bRet = TRUE;
	}

	return bRet;
}

BOOL
JTAPIWrap_IsInitialized()
{
	return gs_bInitialized;
}

void
JTAPIWrap_ClosePlugin()
{
	if (gs_bInitialized) {
		(*gs_Entry.pfnClosePlugin)(gs_hSelf);
	}
}

DWORD
JTAPIWrap_IniReadSection(
	 LPWSTR lpReturnedString,
	 DWORD nSize
)
{
	DWORD	dwRet = 0;

	if (gs_bInitialized) {
		dwRet = (*gs_Entry.pfnIniReadSection)(gs_hSelf, lpReturnedString, nSize);
	}

	return dwRet;
}

int
JTAPIWrap_IniReadInt(
	LPCWSTR lpKeyName,
	int nDefault
)
{
	int	nRet = 0;

	if (gs_bInitialized) {
		nRet = (*gs_Entry.pfnIniReadInt)(gs_hSelf, lpKeyName, nDefault);
	}

	return nRet;

}

DWORD
JTAPIWrap_IniReadString(
	LPCWSTR lpKeyName,
	LPCWSTR lpDefault,
	LPWSTR lpReturnedString,
	DWORD nSize
)
{
	DWORD	dwRet = 0;

	if (gs_bInitialized) {
		dwRet = (*gs_Entry.pfnIniReadString)(gs_hSelf, lpKeyName, lpDefault, lpReturnedString, nSize);
	}

	return dwRet;
}

BOOL
JTAPIWrap_IniReadBool(
	LPCWSTR lpKeyName,
	BOOL fDefault
)
{
	BOOL	bRet = FALSE;

	if (gs_bInitialized) {
		bRet = (*gs_Entry.pfnIniReadBool)(gs_hSelf, lpKeyName, fDefault);
	}

	return bRet;
}


BOOL
JTAPIWrap_IniWriteSection(
	LPCWSTR lpString
)
{
	BOOL	bRet = FALSE;

	if (gs_bInitialized) {
		bRet = (*gs_Entry.pfnIniWriteSection)(gs_hSelf, lpString);
	}

	return bRet;
}

BOOL
JTAPIWrap_IniWriteInt(
	LPCWSTR lpKeyName,
	int nValue
)
{
	BOOL	bRet = FALSE;

	if (gs_bInitialized) {
		bRet = (*gs_Entry.pfnIniWriteInt)(gs_hSelf, lpKeyName, nValue);
	}

	return bRet;
}

BOOL
JTAPIWrap_IniWriteString(
	LPCWSTR lpKeyName,
	LPCWSTR lpString
)
{
	BOOL	bRet = FALSE;

	if (gs_bInitialized) {
		bRet = (*gs_Entry.pfnIniWriteString)(gs_hSelf, lpKeyName, lpString);
	}

	return bRet;
}

BOOL
JTAPIWrap_IniWriteBool(
	LPCWSTR lpKeyName,
	BOOL fValue
)
{
	BOOL	bRet = FALSE;

	if (gs_bInitialized) {
		bRet = (*gs_Entry.pfnIniWriteBool)(gs_hSelf, lpKeyName, fValue);
	}

	return bRet;
}
